"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerativeAi = void 0;
const moment_1 = __importDefault(require("moment"));
const uuid_1 = require("uuid");
const axios_1 = require("axios");
const api_1 = require("../lib/api");
const types_1 = require("../../common/types");
const serialise_graph_1 = require("./serialise_graph");
const sessionContextId = (contextId) => `sirenapi-ai:${contextId}`;
function setContextItem(contextId, message) {
    sessionStorage.setItem(sessionContextId(contextId), JSON.stringify(message));
}
function getContextItem(contextId) {
    const item = sessionStorage.getItem(sessionContextId(contextId));
    return item ? JSON.parse(item) : [];
}
/**
 * Generative AI class. Accessible via `sirenapi.extensions.GenerativeAi`
 * @class
 */
class GenerativeAi {
    constructor() {
        throw new Error('Can not be instantiated');
    }
    /**
     * Send a prompt to the AI
     * @example await sirenapi.extensions.GenerativeAi.sendPrompt('What is 2 * 4?')
     * @param {string} message - The message to send to the AI
     * @param {object} [options] - Additional options
     * @param {string} [options.contextId] - The ID of the conversation context; helps the LLM remember previous messages
     * @returns {Promise<{ data: { timestamp: string; content: string }; contextId: string }>} - The response from the LLM
     */
    static async sendPrompt(message, options) {
        try {
            const contextId = options?.contextId || (0, uuid_1.v4)();
            let previousContext = getContextItem(contextId);
            const newMessage = { user: types_1.User.You, content: message, timestamp: (0, moment_1.default)().toISOString() };
            const currentMessages = [...previousContext, newMessage];
            const response = await api_1.api.post('/chat', { messages: currentMessages });
            previousContext = [...currentMessages, response.data];
            setContextItem(contextId, previousContext);
            return {
                data: response.data,
                contextId
            };
        }
        catch (e) {
            throw new Error(`Failed to get a response: ${e.message}`, { cause: e });
        }
    }
    /**
     * Generate a report from a graph model
     * @example
     * const graphVis = await sirenapi.Visualization.getCurrentVisualizer()
     * const graphModel = graphVis.getGraphModel()
     * await sirenapi.extensions.GenerativeAi.generateGraphReport(graphModel)
     *
     * @param {object} graphModel - The graph model to generate a report from. Retrieved from the `graphBrowserVis.getGraphModel()`
     * @param {object} [options] - Additional options
     * @param {string} [options.focusNodes] - The IDs of the nodes to focus on in the report
     * @returns {Promise<{ content: string }>} - The generated report content in Markdown format
     */
    static async generateGraphReport(graphModel, options) {
        try {
            const body = {
                graph: await (0, serialise_graph_1.serializeGraphModel)(graphModel),
                selection: options?.focusNodes
            };
            const response = await api_1.api.post('/generateReport', body);
            return response.data;
        }
        catch (e) {
            const message = ((0, axios_1.isAxiosError)(e) && e.status === 504)
                ? 'A gateway timeout (504) occurred generating the graph report. Consider querying with a smaller graph or increasing the timeout.'
                : `Failed to generate graph report: ${e.message}`;
            throw new Error(message, { cause: e });
        }
    }
}
exports.GenerativeAi = GenerativeAi;
